"use strict";
// following pseudo code from here :
// https://stackoverflow.com/questions/10657503/find-running-median-from-a-stream-of-integers
// using heap implementation: https://www.npmjs.com/package/@datastructures-js/heap
Object.defineProperty(exports, "__esModule", { value: true });
exports.InfiniteMedian = void 0;
var heap_1 = require("@datastructures-js/heap");
var InfiniteMedian = /** @class */ (function () {
    function InfiniteMedian() {
        this.min = new heap_1.MinHeap();
        this.max = new heap_1.MaxHeap();
        this.initialNodesProcessed = false;
    }
    InfiniteMedian.prototype.insert = function (node) {
        if (!this.initialNodesProcessed) {
            if (this.firstNode === undefined) {
                this.firstNode = node;
                return;
            }
            if (this.secondNode === undefined) {
                this.secondNode = node;
            }
            // process first 2 nodes
            var _a = this.firstNode > this.secondNode
                ? [this.secondNode, this.firstNode]
                : [this.firstNode, this.secondNode], smallerNode = _a[0], biggerNode = _a[1];
            this.max.insert(smallerNode); // O(ln n)
            this.min.insert(biggerNode); // O(ln n)
            this.initialNodesProcessed = true;
            return;
        }
        var maxHeapRoot = this.max.root(); // O(1)
        if (node < maxHeapRoot) {
            this.max.insert(node); // O(ln n)
        }
        else {
            this.min.insert(node); // O(ln n)
        }
        var elementsInMaxheap = this.max.size(); // O(1)
        var elementsInMinheap = this.min.size(); // O(1)
        var difference = Math.abs(elementsInMaxheap - elementsInMinheap);
        if (difference > 1) {
            // balance heaps
            var _b = elementsInMaxheap > elementsInMinheap
                ? [this.max, this.min]
                : [this.min, this.max], biggerSizeheap = _b[0], smallerSizeHeap = _b[1];
            var biggerHeapSizeRoot = biggerSizeheap.extractRoot(); // O(ln n)
            smallerSizeHeap.insert(biggerHeapSizeRoot); // O(ln n)
        }
    };
    InfiniteMedian.prototype.getMedian = function () {
        // edge case when we only have 2 or less points
        if (!this.initialNodesProcessed) {
            if (this.firstNode !== undefined && this.secondNode !== undefined) {
                return (this.firstNode + this.secondNode) / 2;
            }
            return this.firstNode;
        }
        var elementsInMaxheap = this.max.size(); // O(1)
        var elementsInMinheap = this.min.size(); // O(1)
        var maxHeapRoot = this.max.root(); // O(1)
        var minHeapRoot = this.min.root(); // O(1)
        if (elementsInMaxheap === elementsInMinheap) {
            return (maxHeapRoot + minHeapRoot) / 2;
        }
        if (elementsInMaxheap > elementsInMinheap) {
            return maxHeapRoot;
        }
        return minHeapRoot;
    };
    return InfiniteMedian;
}());
exports.InfiniteMedian = InfiniteMedian;
